/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.imp;

import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;
import org.openide.util.*;
import org.openide.util.Lookup.Result;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-farm")
@NbBundle.Messages({
    "ImportNode.name=Import"
})
public class ImportNodeFactory implements NodeFactory {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.fieldimport");

    @Override
    public NodeList<?> createNodes(Project project) {
        // add import root node to project
        FileObject importFolder;
        try {
            importFolder = ImportProcessor.Util.getImportFolder(project);
            DataFolder importDataObject = DataFolder.findFolder(importFolder);
            return NodeFactorySupport.fixedNodeList(new ImportRootNode(importDataObject.getNodeDelegate(), project));
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Error while creating root import node {0}", ex.getMessage());
            Exceptions.printStackTrace(ex);
            return NodeFactorySupport.fixedNodeList();
        }
    }

    private static final class ImportRootNode extends AbstractImportFolderNode {

        private Project project;
        
        public ImportRootNode(Node original, Project project) {
//            super(original, NodeFactorySupport.createCompositeChildren(project, "Projects/nl-cloudfarming-client-import/Nodes"));
            super(project, original, Children.create(new ImportNodeChildFactory(project), true));
            this.project = project;
            setName(Bundle.ImportNode_name());
            setDisplayName(Bundle.ImportNode_name());
        }

        @Override
        public Action[] getActions(boolean context) {
            return new Action[]{new ImportAction(project)};
        }
    }

    private static final class ImportNodeChildFactory extends ChildFactory<Node> {

        private final Project project;
        private static final Result<ImportProcessor> processorResult = Lookup.getDefault().lookupResult(ImportProcessor.class);
        
        public ImportNodeChildFactory(Project project) {
            this.project = project;
            processorResult.addLookupListener(new LookupListener() {
                @Override
                public void resultChanged(LookupEvent ev) {
                    refresh(true);
                }
            });
        }

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            Set<Node> nodes = new HashSet<>();
            for (ImportProcessor importProcessor: processorResult.allInstances()){
                nodes.add(importProcessor.getImportNode(project));
            }
            toPopulate.addAll(nodes);
            return true;
        }
    }
}
