/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.imp;

import java.util.Set;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Node;

/**
 * class describing which files in the import source should be displayed
 * @author Timon Veenstra <monezz@gmail.com>
 */
public abstract class ImportSource {
 
    
    /**
     * Get the files which must be used as import source
     * @return 
     */
    public abstract Set<FileObject> getFiles();
    
    /**
     * Looks in the provided node for an instance of {@link importSource}
     * if found returns true else false
     * @param node
     * @return 
     */
    public static boolean accept(Node node) {
            // filter out secondary files:
            ImportSource is = null;
            // It is not adviced to lookup a dataobject on a node
            // (see http://netbeans.org/bugzilla/show_bug.cgi?id=199391)
            // instead we lookup the fileobject and load the dataobject
            // from the fileobject.
            // if the dataobject contains a shape import source in its lookup
            // it is the primary object and should be shown.
            // secondary files are filtered this way
            FileObject fo = node.getLookup().lookup(FileObject.class);
            if(fo != null){
                try {
                    DataObject dataObject = DataObject.find(fo);
                    if (dataObject != null) {
                        is = (ImportSource) dataObject.getLookup().lookup(ImportSource.class);
                    }
                } catch (DataObjectNotFoundException ex) {
                    //rejecting
                }
            }
            return (is != null);
        }
}
